package org.example;
import org.example.abilities.DoubleAttack;
import org.example.abilities.SpecialAbility;
import org.example.items.Item;

import java.util.Random;
import java.util.Scanner;

public class BeatThemAllGame {

    /**
     * Affichage du titre en ASCII
     */
    public static void displayTitle() {
        System.out.println("**************************************************");
        System.out.println("*                 BEAT THEM ALL                  *");
        System.out.println("**************************************************");
        System.out.println("          ____        _   _   _                  ");
        System.out.println("         |  _ \\      | | | | | |                 ");
        System.out.println("         | |_) | __ _| |_| |_| | ___             ");
        System.out.println("         |  _ < / _` | __| __| |/ _ \\            ");
        System.out.println("         | |_) | (_| | |_| |_| |  __/            ");
        System.out.println("         |____/ \\__,_|\\__|\\__|_|\\___|            ");
        System.out.println();
    }

    /**
     * Affichage du message de mort
     */
    public static void displayGameOver() {
        System.out.println("\n=== VOUS ÊTES MORT ===");
        System.out.println("      _____");
        System.out.println("     /     \\");
        System.out.println("    |       |");
        System.out.println("    |  RIP  |");
        System.out.println("    |       |");
        System.out.println("    |       |");
        System.out.println("    |       |");
        System.out.println("    |_______|");
        System.out.println("\nMerci d'avoir joué !");
    }


    /**
     * Sélection du héros
     * @param scanner Object Scanner qui permet de lire l'entrée standard
     * @return Retourne le héros sélectionné
     */
    public static Hero chooseHero(Scanner scanner) {
        System.out.println("Choisissez votre héros :");
        System.out.println("1. Guerrier (Double Attaque)");
        System.out.println("2. Mage (Réduit les PV des ennemis de moitié)");
        System.out.println("3. Voleur (Tentative de vol d'objets)");
        System.out.println("4. Archer (Attaque à distance avant le combat)");
        System.out.println("5. Berserker (Plus il est blessé, plus il frappe fort)");
        System.out.print("Votre choix : ");
        int choice = scanner.nextInt();
        scanner.nextLine(); // Consommer le retour à la ligne
        System.out.print("Entrez le nom de votre héros : ");
        String name = scanner.nextLine();

        switch (choice) {
            case 1:
                return new Hero(name, 80, 20, 10, new DoubleAttack());
            case 2:
                return new Hero(name, 60, 15, 5, new HalveEnemyHP());
            case 3:
                return new Hero(name, 70, 18, 7, new StealItem());
            case 4:
                return new Hero(name, 75, 17, 6, new PreemptiveStrike());
            case 5:
                return new Hero(name, 100, 22, 5, new Rage());
            default:
                System.out.println("Choix invalide, Guerrier sélectionné par défaut.");
                return new Hero(name, 150, 20, 10, new DoubleAttack());
        }
    }

    // Capacités spéciales supplémentaires (a voir si on garde)
    public static class PreemptiveStrike implements SpecialAbility {
        @Override
        public void activate(Character user, Character target) {
            int damage = user.attackPower;
            System.out.println(user.name + " tire une flèche sur " + target.name + " avant le combat !");
            target.takeDamage(damage);
        }
    }

    public static class Rage implements SpecialAbility {
        @Override
        public void activate(Character user, Character target) {
            int rageBonus = (user.maxHP - user.currentHP) / 10;
            user.attackPower += rageBonus;
            System.out.println(user.name + " entre en rage, augmentant son attaque de " + rageBonus + " points !");
        }
    }

    // ennemi aléatoire
    public static Enemy generateRandomEnemy() {
        int enemyType = new Random().nextInt(4);
        switch (enemyType) {
            case 0:
                return new Enemy("Brigand", 80, 15, 5, null);
            case 1:
                return new Enemy("Sorcier", 90, 10, 5, new WeaknessSpell());
            case 2:
                return new Enemy("Gangster", 70, 12, 4, null);
            case 3:
                return new Enemy("Catcheur", 150, 10, 8, null);
            default:
                return new Enemy("Brigand", 80, 15, 5, null);
        }
    }

    // pareil pour uin objet
    public static Item getRandomItem() {
        int itemType = new Random().nextInt(2);
        switch (itemType) {
            case 0:
                return new HealthPotion();
            case 1:
                return new AttackBoost();
            default:
                return new HealthPotion();
        }
    }

    public static class HalveEnemyHP implements SpecialAbility {
        @Override
        public void activate(Character user, Character target) {
            System.out.println(user.name + " réduit de moitié les PV de " + target.name + " !");
            target.currentHP /= 2;
        }
    }

    public static class StealItem implements SpecialAbility {
        @Override
        public void activate(Character user, Character target) {
            System.out.println(user.name + " tente de voler un objet à " + target.name + " !");
            // Simplification : L'ennemi n'a pas d'objets à voler
            System.out.println("Mais " + target.name + " n'a rien à voler !");
        }
    }

    // pouvoir des ennemis (on pourra en ajouter ou en enelver)
    public static class WeaknessSpell implements SpecialAbility {
        @Override
        public void activate(Character user, Character target) {
            System.out.println(user.name + " lance un sort de faiblesse sur " + target.name + " !");
            target.attackPower = Math.max(target.attackPower - 5, 0);
            System.out.println(target.name + " voit son attaque réduite à " + target.attackPower + " !");
        }
    }

    public static class HealthPotion implements Item {
        @Override
        public void applyEffect(Hero hero) {
            int healAmount = 50;
            hero.currentHP = Math.min(hero.currentHP + healAmount, hero.maxHP);
            System.out.println("Vous récupérez " + healAmount + " PV. (PV actuels : " + hero.currentHP + ")");
        }

        @Override
        public String getName() {
            return "Potion de Santé";
        }
    }

    public static class AttackBoost implements Item {
        @Override
        public void applyEffect(Hero hero) {
            int boostAmount = 5;
            hero.attackPower += boostAmount;
            System.out.println("Votre attaque augmente de " + boostAmount + " points ! (Attaque actuelle : " + hero.attackPower + ")");
        }

        @Override
        public String getName() {
            return "Potion de Puissance";
        }
    }

}

